import logging

import botocore

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def get_regions(self):
    logger.info(" ---Inside utils :: get_regions()--- ")
    """Summary

    Returns:
        TYPE: Description
    """

    client = self.session.client('ec2', region_name='us-east-1')
    region_response = {}
    try:
        region_response = client.describe_regions()
    except botocore.exceptions.ClientError as error:
        if error.response['Error']['Code'] == 'AuthFailure':
            logger.error(f" AccessKey credentails not found here: {error}")
            exit(1)
    except botocore.exceptions.NoCredentialsError as e:
        logger.error(f" Unable to locate credentials: {e} ")
        exit(1)

    regions = [region['RegionName'] for region in region_response['Regions']]
    return regions


# returns the list of lambda functions
def list_lambda_functions(client) -> dict:
    """
    :param client:
    :param self:
    :return:
    """
    logger.info(" ---Inside lambdafn.utils :: list_lambda_functions")

    function_lst = []

    marker = ''
    while True:
        if marker == '' or marker is None:
            response = client.list_functions()
        else:
            response = client.list_functions(
                Marker=marker
            )
        for fn in response['Functions']:
            function_lst.append(fn)

        try:
            marker = response['NextMarker']
            if marker == '':
                break
        except KeyError:
            break

    return function_lst




import logging

import botocore

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def asg_elb_healthcheck_required(self, regions) -> dict:
    """
    :param self:
    :return dict:
    """
    logger.info(" ---Inside autoscaling :: asg_elb_healthcheck_required()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "AutoScaling Group ELB health check required"
    description = "Checks if ELB health check is enabled on autoscaling group"
    resource_type = "Auto Scaling Group"
    risk_level = 'Medium'

    # regions = self.session.get_available_regions('autoscaling')

    for region in regions:
        try:
            client = self.session.client('autoscaling', region_name=region)
            marker = ''
            while True:
                if marker == '':
                    response = client.describe_auto_scaling_groups(
                        MaxRecords=100
                    )
                else:
                    response = client.describe_auto_scaling_groups(
                        NextToken=marker,
                        MaxRecords=100
                    )
                for asg in response['AutoScalingGroups']:
                    if asg['HealthCheckType'] != 'ELB':
                        result = False
                        offenders.append(asg['AutoScalingGroupName'])
                        failReason = 'Health check type in autoscaling group is not ELB'

                try:
                    marker = response['NextToken']
                    if marker == '':
                        break
                except KeyError:
                    break
        except botocore.exceptions.ClientError:
            pass

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level
    }



import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks the compliance.py for s3 bucket versioning enabled
def s3_event_notifications_enabled(self):
    """

    :param self:
    :return dict:
    """
    logger.info(" ---Inside OBP MOnitoring :: s3_event_notificationsenabled()")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "S3 event notifications enabled"
    description = "Checks if Amazon S3 Events Notifications are enabled on an S3 bucket."
    resource_type = "S3 Buckets"
    risk_level = 'Medium'

    client = self.session.client('s3')
    response = client.list_buckets()

    for bucket in response['Buckets']:
        bucket_name = bucket['Name']

        try:
            resp = client.get_bucket_notification_configuration(
                Bucket=bucket_name,
            )
            #print(resp)
            del resp['ResponseMetadata']
            notifications_enabled = bool(resp)
            if notifications_enabled == False:
                raise KeyError
            
        except KeyError:
            result = False
            failReason = "Notifications are turned off for S3 buckets."
            offenders.append(bucket_name)
            continue

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level
    }



import botocore
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance.py for ec2 instance detailed monitoring enabled
def ec2_instance_detailed_monitoring_enabled(self, regions) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside ec2 :: ec2_instance_detailed_monitoring_enabled")

    result = True
    failReason = ''
    offenders = []
    compliance_type = "EC2 instance detailed monitoring enabled"
    description = "Checks if detailed monitoring is enabled for EC2 instances."
    resource_type = "EC2 Instance"
    risk_level = 'Low'

    # regions = self.session.get_available_regions('ec2')

    for region in regions:
        try:
            client = self.session.client('ec2', region_name=region)
            marker = ''
            while True:
                response = client.describe_instances(
                    MaxResults=1000,
                    NextToken=marker
                )
                if len(response['Reservations']) > 0:
                    for reservation in response['Reservations']:
                        for instance in reservation['Instances']:
                            monitoring = instance['Monitoring']['State']
                            if monitoring != 'enabled':
                                result = False
                                failReason = "Monitoring is not enabled in instances"
                                offenders.append(region+': '+instance['InstanceId'])

                try:
                    marker = response['NextToken']
                    if marker == '':
                        break
                except KeyError:
                    break
        except botocore.exceptions.ClientError as e:
            logger.error('Something went wrong with region {}: {}'.format(region, e))

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level
    }



from boto3 import session
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


__author__ = 'Klera DevOps'
__version__ = '0.0.1'


class aws_client:
    def __init__(self, **kwargs):
        if 'aws_access_key_id' in kwargs.keys() and 'aws_secret_access_key' in kwargs.keys():
            self.session = session.Session(
                aws_access_key_id=kwargs['aws_access_key_id'],
                aws_secret_access_key=kwargs['aws_secret_access_key'],
            )
        elif 'profile_name' in kwargs.keys():
            self.session = session.Session(profile_name=kwargs['profile_name'])

    from .lambda_concurrency_check import lambda_concurrency_check
    from .s3_event_notifications_enabled import s3_event_notifications_enabled
    from .asg_elb_healthcheck_required import asg_elb_healthcheck_required
    from .ec2_instance_detailed_monitoring_enabled import ec2_instance_detailed_monitoring_enabled
    from .guard_duty_enabled import guard_duty_enabled
    from .lambda_dlq_check import lambda_dlq_check
    from .rds_enhanced_monitoring_enabled import rds_enhanced_monitoring_enabled
    from .utils import get_regions

    def get_compliance(self) -> list:
        """
        :return:
        """
        regions = self.get_regions()
        compliance = [
            self.lambda_concurrency_check(regions),
            self.s3_event_notifications_enabled(),
            self.asg_elb_healthcheck_required(regions),
            self.ec2_instance_detailed_monitoring_enabled(regions),
            self.guard_duty_enabled(regions),
            self.lambda_dlq_check(regions),
            self.rds_enhanced_monitoring_enabled(regions),
        ]

        return compliance
